﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Security.Contracts.Interfaces;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;
using System.ServiceModel;
using HeuristicLab.Hive.Server.DataAccess;


namespace HeuristicLab.Hive.Server.Core {
  public class ServerConsoleFacade : IServerConsoleFacade {
    private ISlaveManager slaveManager = ServiceLocator.GetSlaveManager();

    private IJobManager jobManager = ServiceLocator.GetJobManager();

    private IHivePermissionManager secMan = ServiceLocator.GetHivePermissionManager();

    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    public Guid sessionID = Guid.Empty;

    public ServerConsoleFacade() {
    }

    public Response Login(string username, string password) {
      Response resp = new Response();

      sessionID = secMan.Login(username, password);
      if (sessionID == Guid.Empty) {
        resp.Success = false;
        resp.StatusMessage = ApplicationConstants.RESPONSE_SERVERCONSOLE_LOGIN_FAILED;
      } else {
        resp.Success = true;
        resp.StatusMessage =
          ApplicationConstants.RESPONSE_SERVERCONSOLE_LOGIN_SUCCESS;
      }
      return resp;
    }


    public ResponseList<ClientDto> GetAllClients() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessClients", sessionID, Guid.Empty);
        return slaveManager.GetAllClients();
      }
    }

    public ResponseList<ClientGroupDto> GetAllClientGroups() {
      using (contextFactory.GetContext(false)) {
        //secMan.Authorize("AccessClientGroup", sessionID, Guid.Empty);
        return slaveManager.GetAllClientGroups();
      }
    }

    public ResponseList<UpTimeStatisticsDto> GetAllUpTimeStatistics() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessStatistics", sessionID, Guid.Empty);
        return slaveManager.GetAllUpTimeStatistics();
      }
    }

    public ResponseObject<ClientGroupDto> AddClientGroup(ClientGroupDto clientGroup) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AddClientGroup", sessionID, Guid.Empty);
        return slaveManager.AddClientGroup(clientGroup);
      }
    }

    public Response AddResourceToGroup(Guid clientGroupId, ResourceDto resource) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AddResource", sessionID, Guid.Empty);
        return slaveManager.AddResourceToGroup(clientGroupId, resource);
      }
    }

    public Response DeleteResourceFromGroup(Guid clientGroupId, Guid resourceId) {
      using (contextFactory.GetContext()) {
        return slaveManager.DeleteResourceFromGroup(clientGroupId, resourceId);
      }
    }

    public ResponseList<JobDto> GetAllJobs() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, Guid.Empty);
        return jobManager.GetAllJobs();
      }
    }

    public ResponseList<JobDto> GetAllJobsWithFilter(State jobState, int offset, int count) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, Guid.Empty);
        return jobManager.GetAllJobsWithFilter(jobState, offset, count);
      }
    }

    public ResponseObject<JobDto> GetJobById(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, jobId);
        return jobManager.GetJobById(jobId);
      }
    }

    public ResponseObject<JobDto> GetJobByIdWithDetails(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, jobId);
        return jobManager.GetJobByIdWithDetails(jobId);
      }
    }

    public ResponseObject<JobDto> AddNewJob(SerializedJob job) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AddJob", sessionID, job.JobInfo.Id);
        return jobManager.AddNewJob(job);
      }
    }

    public ResponseObject<JobDto> GetLastJobResultOf(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobResults", sessionID, jobId);
        return jobManager.GetLastJobResultOf(jobId);
      }
    }

    public ResponseObject<SerializedJob> GetLastSerializedJobResultOf(Guid jobId, bool requested, bool snapshot) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobResults", sessionID, jobId);
        return jobManager.GetLastSerializedJobResultOf(jobId, requested, snapshot);
      }
    }

    public ResponseObject<JobResultList> GetAllJobResults(IEnumerable<Guid> jobIds) {
      using (contextFactory.GetContext(false)) {
        //secMan.Authorize("AccessJobResults", sessionID, jobId); skip authorization
        return jobManager.GetAllJobResults(jobIds);
      }
    }

    public Response RemoveJob(Guid jobId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("RemoveJob", sessionID, jobId);
        return jobManager.RemoveJob(jobId);
      }
    }

    public Response RequestSnapshot(Guid jobId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AccessJobResults", sessionID, jobId);
        return jobManager.RequestSnapshot(jobId);
      }
    }

    public Response AbortJob(Guid jobId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("AbortJob", sessionID, Guid.Empty);
        return jobManager.AbortJob(jobId);
      }
    }

    public ResponseObject<ClientGroupDtoList> GetAllGroupsOfResource(Guid resourceId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessUserGroup", sessionID, Guid.Empty);
        return slaveManager.GetAllGroupsOfResource(resourceId);
      }
    }

    public Response DeleteClientGroup(Guid clientGroupId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("DeleteClientGroup", sessionID, Guid.Empty);
        return slaveManager.DeleteClientGroup(clientGroupId);
      }
    }

    public ResponseList<ProjectDto> GetAllProjects() {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessProjects", sessionID, Guid.Empty);
        return jobManager.GetAllProjects();
      }
    }

    public Response CreateProject(ProjectDto project) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("CreateProjects", sessionID, Guid.Empty);
        return jobManager.CreateProject(project);
      }
    }

    public Response ChangeProject(ProjectDto project) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("ChangeProjects", sessionID, Guid.Empty);
        return jobManager.ChangeProject(project);
      }
    }

    public Response DeleteProject(Guid projectId) {
      using (contextFactory.GetContext()) {
        secMan.Authorize("DeleteProjects", sessionID, projectId);
        return jobManager.DeleteProject(projectId);
      }
    }

    public ResponseList<JobDto> GetJobsByProject(Guid projectId) {
      using (contextFactory.GetContext(false)) {
        secMan.Authorize("AccessJobs", sessionID, Guid.Empty);
        return jobManager.GetJobsByProject(projectId);
      }
    }

    public ResponseList<AppointmentDto> GetUptimeCalendarForResource(Guid guid) {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetUptimeCalendarForResource(guid);
      }
    }

    public Response SetUptimeCalendarForResource(Guid guid, IEnumerable<AppointmentDto> appointments, bool isForced) {
      using (contextFactory.GetContext()) {
        return slaveManager.SetUptimeCalendarForResource(guid, appointments, isForced);
      }
    }

    public ResponseObject<JobDto> AddJobWithGroupStrings(SerializedJob job, IEnumerable<string> resources) {
      using (contextFactory.GetContext()) {
        return jobManager.AddJobWithGroupStrings(job, resources);
      }
    }
  }
}
